@@ -6,7 +6,7 @@ prepareDSList <- function(pathData, pathDescription)
     stopifnot(file.info(pathDescription)$isdir)
     dsList <- readDSListFromXML(paste(pathDescription, "contents.xml", sep="/"))
     dsList <- data.frame(pathData, pathDescription, dsList)
-    dsList$available <- checkAvailability(dsList, seq.int(length.out=nrow(dsList)))
+    dsList$available <- getAvailable(dsList, seq.int(length.out=nrow(dsList)), asLogical=TRUE)
     OK <- checkConsistency(dsList)
     if (OK) {
         cat(sum(dsList$available), "data sets available\n")
@@ -17,6 +17,7 @@ prepareDSList <- function(pathData, pathDescription)
 dsSearch <- function(dsList, id, searchField=c("identification", "fullName", "dirName", "files"),
             searchType=c("exact", "prefix", "suffix", "anywhere"), caseSensitive=FALSE)
 {
+    stopifnot(is.data.frame(dsList))
     if (is.numeric(id)) {
         stopifnot(all(id >= 1))
         stopifnot(all(id <= nrow(dsList)))
@@ -24,7 +25,7 @@ dsSearch <- function(dsList, id, searchField=c("identification", "fullName", "di
         ind <- id
         if (length(ind) == 0) {
             cat("no data set found for", id, "\n")
-            return(NULL)
+            return(invisible(NULL))
         }
         out <- data.frame(ind=ind, identification=dsList$identification[ind], stringsAsFactors=FALSE)
     } else {
@@ -50,7 +51,7 @@ dsSearch <- function(dsList, id, searchField=c("identification", "fullName", "di
         }
         if (length(ind) == 0) {
             cat("no data set found for", id, "\n")
-            return(NULL)
+            return(invisible(NULL))
         }
         out <- data.frame(ind=ind, identification=dsList$identification[ind], stringsAsFactors=FALSE)
         if (searchField == "fullName") {
@@ -71,18 +72,19 @@ getData <- function(x)
 
 dsRead <- function(dsList, id, responseName=NULL, originalNames=TRUE, deleteUnused=TRUE, keepContents=FALSE)
 {
+    stopifnot(is.data.frame(dsList))
     id <- getIndex(dsList, id)
-    if (!dsList$available[id]) {
-        cat("the required data set is not available\n")
-        return(NULL)
-    }
     identification <- dsList$identification[id]
     functionsFile <- paste(dsList$pathDescription[id], "/functions.R", sep="")
     if (file.access(functionsFile) == 0) {
         source(functionsFile, local=TRUE)
     }
     source(paste(dsList$pathDescription[id], "/scripts/", identification, ".R", sep=""), local=TRUE)
-    dat <- getData(paste(dsList$pathData[id], dsList$dirName[id], sep="/"))
+    result <- try( dat <- getData(paste(dsList$pathData[id], dsList$dirName[id], sep="/")) )
+    if (inherits(result, "try-error")) {
+        cat("An error occured, when reading the data.\n")
+        return(invisible(NULL))
+    }
     rm(getData)
     row.names(dat) <- 1:nrow(dat)
     if (ncol(dat) != dsList$originalColsNumber[id]) {
@@ -122,19 +124,59 @@ dsRead <- function(dsList, id, responseName=NULL, originalNames=TRUE, deleteUnus
     dat
 }
 
+getAvailable <- function(dsList, id=NULL, asLogical=FALSE)
+{
+    stopifnot(is.data.frame(dsList))
+    if (is.null(id)) {
+        id <- seq.int(length.out=nrow(dsList))
+    }
+    if (is.character(id)) {
+        id <- match(id, dsList$identification)
+    }
+    out <- rep(NA, times=length(id))
+    for (i in id) {
+        files <- split.comma(dsList$files[i])
+        files <- paste(dsList$pathData[i], dsList$dirName[i], files, sep="/")
+        out[i] <- all(file.access(files) == 0)
+    }
+    if (asLogical) {
+        out
+    } else {
+        dsList$identification[id][out]
+    }
+}
+
 runCommand <- function(dsList, id, command, fileName)
 {
     setwd(paste(dsList$pathData[id], dsList$dirName[id], sep="/"))
-    url <- readLines(fileName)
-    target <- basename(url)
-    for (i in which(file.access(target) == -1)) {
-        system(paste(command, url[i]))
+    links <- readLines(fileName)
+    isComment <- grepl("^ *#", links)
+    address <- links[!isComment]
+    file <- basename(address)
+    doDownload <- file.access(file) == -1
+    cat("\n")
+    for (i in seq.int(along.with=address)) {
+        if (doDownload[i]) {
+            system(paste(command, address[i]))
+            cat("\n")
+        } else {
+            cat("File", file[i], "already exists\n")
+            cat("\n")
+        }
+    }
+    if (any(isComment)) {
+        cat("The data set has the following downloading comments attached.\n")
+        cat("\n")
+        cat(paste(links[isComment], "\n", sep=""))
+        cat("\n")
     }
 }
 
 dsDownload <- function(dsList, id, command, fileName)
 {
+    stopifnot(is.data.frame(dsList))
     id <- getIndex(dsList, id)
+    if (is.null(id)) return(invisible(NULL))
     current <- getwd()
     out <- try( runCommand(dsList, id, command, fileName) )
     setwd(current)
